//
// Find a strong random entropy source, depending on the detected platform.
// WARNING TO DEVELOPERS: This will fail on some systems, but do NOT use
// Crypto.Util.randpool.RandomPool as a fall-back.  RandomPool will happily run
// with very little entropy, thus _silently_ defeating any security that
// Paramiko attempts to provide.  (This is current as of PyCrypto 2.0.1).
// See http://www.lag.net/pipermail/paramiko/2008-January/000599.html
// and http://www.lag.net/pipermail/paramiko/2008-April/000678.html
//

/*
  Wrapper around RandomPool guaranteeing strong random numbers.
    
  Crypto.Util.randpool.RandomPool will silently operate even if it is seeded
  with little or no entropy, and it provides no prediction resistance if its
  state is ever compromised throughout its runtime.  It is also not thread-safe.

  This wrapper augments RandomPool by XORing its output with random bits from
  the operating system, and by controlling access to the underlying
  RandomPool using an exclusive lock.
*/
paramikojs.randpool = function(instance) {
  if (!instance) {
    instance = new crypto.random.Random();
  }
  this.randpool = instance;
  this.entropy = sys.platform == 'win32' ? paramikojs.rng_win32.open_rng_device() : paramikojs.rng_posix.open_rng_device();

  this.randpool.flush();
  this.entropy.randomize();
}

paramikojs.randpool.prototype = {
  stir : function(s) {
    this.randpool.flush();
    this.entropy.randomize();
  },

  randomize : function(N) {
    this.randpool.flush();
    this.entropy.randomize();
  },

  add_event : function(s) {
    this.randpool.flush();
  },

  get_bytes : function(N, dontFlush) {
    var randpool_data = this.randpool.read(N, dontFlush);
    var entropy_data = this.entropy.read(N, dontFlush);
    var result = "";
    for (var x = 0; x < randpool_data.length; ++x) {
      result += String.fromCharCode(randpool_data.charCodeAt(x) ^ entropy_data.charCodeAt(x));
    }
    return result;
  }
};
